﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using VoIPserviceCheckerWP75.Utils;

namespace VoIPserviceCheckerWP75.Modell
{
    public class CommunitcatorUdp
    {
        private Socket clientSocket;

        private DnsEndPoint remoteIpEndPoint;
        public event EventHandler<GenericEventArgs<bool>> ConnectCompleted;
        public event EventHandler<GenericEventArgs<string>> MessageReceived;
        public event EventHandler NoResponese;
        
        private const int maxBufferSize = 80000;
        private ManualResetEvent threadcontrol;

        private static int Timoutconnection = 2000;


        public CommunitcatorUdp()
        {
           
            threadcontrol = new ManualResetEvent(false);
        }

        public bool Connect(string serverAddress)
        {
            clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            clientSocket.ReceiveBufferSize = maxBufferSize;
            clientSocket.SendBufferSize = maxBufferSize;

            //IPAddress ip;
            //if (!IPAddress.TryParse(serverAddress, out ip))
            //{
            //    throw new FormatException("Invalid ip-adress fomat.");
            //}
            
            remoteIpEndPoint = new DnsEndPoint(serverAddress, 5060);

            SocketAsyncEventArgs socketEventArg = new SocketAsyncEventArgs()
            {
                RemoteEndPoint = remoteIpEndPoint
            };
            socketEventArg.SetBuffer(new byte[] { 0 }, 0, 1);
            socketEventArg.Completed += OnConncetCompleted;
            clientSocket.ConnectAsync(socketEventArg);
            threadcontrol.Reset();
            if (threadcontrol.WaitOne(Timoutconnection))
            {
                return true;

            }
            return false;
        }

        public void CloseConnection()
        {
            try
            {
                clientSocket.Close();
            }
            catch (Exception)
            {}
        }

        private void OnConncetCompleted(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success)
            {
                OnConnectionCompleted(true);
                threadcontrol.Set();
                StartReceiving();
                //sikertelen
            }
            else
            {
                OnConnectionCompleted(false);
                //"Socket client can't Conncet to the server!" + System.Environment.NewLine + e.SocketError.ToString());
            }
            try { e.Dispose(); }
            catch { }
        }


        public void StartReceiving()
        {
            try
            {
                SocketAsyncEventArgs socketEventArg = new SocketAsyncEventArgs();
                socketEventArg.RemoteEndPoint = new IPEndPoint(IPAddress.Any, 5780);
                socketEventArg.SetBuffer(new Byte[maxBufferSize], 0, maxBufferSize);
                socketEventArg.Completed += NetworkOperationCompleted;
                threadcontrol.Reset();

                if (!clientSocket.ReceiveAsync(socketEventArg))
                {
                    socketEventArg.UserToken = true;
                    NetworkOperationCompleted(this, socketEventArg);
                }
                if (!threadcontrol.WaitOne(Timoutconnection))
                {
                    OnNoResponese();
                }
            }
            catch (Exception)
            {

                OnNoResponese();
            } 
        }

    

        void NetworkOperationCompleted(object sender, SocketAsyncEventArgs e)
        {
            switch (e.LastOperation)
            {
                case SocketAsyncOperation.Send:
                case SocketAsyncOperation.SendTo:
                    OnSendCompleted(this, e);
                    break;
                case SocketAsyncOperation.Receive:
                case SocketAsyncOperation.ReceiveFrom:
                    //  OnReceiveCompleted(this, e);
                    OnReceiveCompleted(e);
                    break;
            }
        }


        void OnReceiveCompleted(SocketAsyncEventArgs e)
        {
            threadcontrol.Set();
            if (e.LastOperation != SocketAsyncOperation.ReceiveFrom && e.LastOperation != SocketAsyncOperation.Receive)
                return;


            if (e.BytesTransferred > 0 && e.SocketError == SocketError.Success)
            {

               string response = Encoding.UTF8.GetString(e.Buffer, e.Offset, e.BytesTransferred);
               response = response.Trim('\0');
                OnMessageReceived(response);
            }
            else
            {
                //   OnCloseConnection();
            }
            try
            {
                e.Dispose();
            }
            catch {}
        }

        private void OnMessageReceived(string response)
        {
            var handler = MessageReceived;
            if (handler!=null)
            {
                MessageReceived(this, new GenericEventArgs<string>(response));
            }
        }

        public bool Send_Bytes(byte[] buffer)
        {
            if (buffer == null)
                return false;

            SocketAsyncEventArgs socketEventArg = new SocketAsyncEventArgs();
            socketEventArg.Completed += NetworkOperationCompleted;

            socketEventArg.SetBuffer(buffer, 0, buffer.Length);
            socketEventArg.RemoteEndPoint = remoteIpEndPoint;

            if (clientSocket.Connected)
            {
                clientSocket.SendAsync(socketEventArg);

                threadcontrol.Reset();
                if (threadcontrol.WaitOne(Timoutconnection))
                {
                    return true;
                }
             
            }
            else
            {
                //     OnCloseConnection();
            }
            return false;
        }

        void OnSendCompleted(object sender, SocketAsyncEventArgs e)
        {

            threadcontrol.Set();
            if (e.SocketError != SocketError.Success)
            {
                //   OnCloseConnection();
            }


            StartReceiving();

            e.Dispose();
        }



        void OnConnectionCompleted(bool isSuccess)
        {
            var handler = ConnectCompleted;
            if (handler != null)
            {
                ConnectCompleted(this, new GenericEventArgs<bool>(isSuccess));
            }
        }

        private void OnNoResponese()
        {
            var handler = NoResponese;
            if (handler != null)
            {
                NoResponese(this, new EventArgs());
            }
        }

    }
}
