﻿using System;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using VoIPserviceCheckerWP75.Utils;

namespace VoIPserviceCheckerWP75.Modell
{
    public class SipRegister
    {

        private CommunitcatorUdp communitcator;
        private string mylocalIp;
        public event EventHandler<GenericEventArgs<RegistrationResult>> RegistrationCompleted;
        private string userName;
        private string authname;
        private string password;
        private string sipProviderhost;
        private bool loginAlreadyTried;
        private string tag = "hetfgee0";
        private string branch = "z9hG4bK5b5077e0-e26f-4a50-bf16-10b5b22b591c";

        public SipRegister()
        {
            communitcator = new CommunitcatorUdp();
            communitcator.MessageReceived += new EventHandler<GenericEventArgs<string>>(communitcator_MessageReceived);
            communitcator.NoResponese += new EventHandler(communitcator_NoResponese);
            mylocalIp = GetMyip();
        }

        void communitcator_NoResponese(object sender, EventArgs e)
        {
            OnRegistrationCompleted(RegistrationResult.Notfound);
        }

        void communitcator_MessageReceived(object sender, GenericEventArgs<string> e)
        {
            if (e.Item.ToLowerInvariant().Contains("authentication required") || e.Item.ToLowerInvariant().Contains("401 unauthorized"))
            {
                if (loginAlreadyTried)
                {
                    OnRegistrationCompleted(RegistrationResult.FoundNotRegistered);
                    return;
                }
                loginAlreadyTried = true;
                string authPrefix = e.Item.ToLowerInvariant().Contains("407 proxy") ? "Proxy-" : "";
                string sipmsg = CreateRegisterMsgWithAuth(e.Item, authPrefix);
                Byte[] sendBytes = Encoding.UTF8.GetBytes(sipmsg);
                if (!communitcator.Send_Bytes(sendBytes))
                {
                    OnRegistrationCompleted(RegistrationResult.Notfound);
                    //timeout not found
                    return;
                }
            }
            else if (e.Item.ToLowerInvariant().Contains("200 ok"))
            {
                OnRegistrationCompleted(RegistrationResult.FoundRegistered);
            }
            else if (e.Item.ToLowerInvariant().Contains("sip"))
            {
                OnRegistrationCompleted(RegistrationResult.FoundNotRegistered);
            }
            else
            {
                OnRegistrationCompleted(RegistrationResult.Notfound);
            }
        }



        public void RegistertoPbx(string userName, string authName, string password, string host)
        {
            ThreadPool.QueueUserWorkItem(o =>
            {
                loginAlreadyTried = false;

                if (mylocalIp == null)
                {
                    //not connected to a network
                }
                this.userName = userName;
                this.authname = authName;
                this.password = password;
                sipProviderhost = host;

                if (communitcator.Connect(host))
                {
                    string sipmsg = CreateRegisterMsg();
                    Byte[] sendBytes = Encoding.UTF8.GetBytes(sipmsg);
                    if (!communitcator.Send_Bytes(sendBytes))
                    {
                        //timeout not found
                        return;
                    }
                }
                else
                {
                    OnRegistrationCompleted(RegistrationResult.Notfound);
                }
            });
        }

        private void OnRegistrationCompleted(RegistrationResult result)
        {
            var handler = RegistrationCompleted;
            if (handler != null)
                RegistrationCompleted(this, new GenericEventArgs<RegistrationResult>(result));
            communitcator.CloseConnection();
        }


        private string CreateRegisterMsg()
        {
            string message = "REGISTER sip:" + sipProviderhost + ":5060 SIP/2.0\r\n"
                + "Via:SIP/2.0/UDP " + mylocalIp + ":5780;branch=" + branch + ";rport\r\n"
                + "To: " + userName + "<sip:" + userName + "@" + sipProviderhost + ":5060>\r\n"
                + "From: " + userName + "<sip:" + userName + "@" + sipProviderhost + ":5060>;tag=" + tag + "\r\n"
                + "Call-ID: gdhisemewofqcjxysiwjjnrtfdwsfcmykvspmdxfnpllcrdsyi\r\n"
                + "CSeq: 1 REGISTER\r\n"
                + "Contact: <sip:" + userName + "@" + mylocalIp + ":5780>\r\n"
                + "Expires: 3600\r\n"
                + "Supported: 100rel\r\n"
                + "Allow: INVITE, ACK, CANCEL, OPTIONS, BYE\r\n"
                + "Max-Forwards: 70\r\n"
                + "Content-Length: 0\r\n"
                + "\r\n";
            return message;
        }


        private string CreateRegisterMsgWithAuth(string receivedMessage, string authPrefix)
        {
            string nonce = GetNonce(receivedMessage);
            string realm = GetRealm(receivedMessage);
            string authresponse = GetAuthResponse(nonce, realm);

            string message = "REGISTER sip:" + sipProviderhost + ":5060 SIP/2.0\r\n"
              + "Via:SIP/2.0/UDP " + mylocalIp + ":5780;branch=" + "z9hG4bK78b5493d-cb3c-4ad6-a435-1808b74a262d" + ";rport\r\n"
              + "To: " + userName + "<sip:" + userName + "@" + sipProviderhost + ":5060>\r\n"
              + "From: " + userName + "<sip:" + userName + "@" + sipProviderhost + ":5060>;tag=" + tag + "\r\n"
              + "Call-ID: gdhisemewofqcjxysiwjjnrtfdwsfcmykvspmdxfnpllcrdsyi\r\n"
              + "CSeq: 2 REGISTER\r\n"
              + "Contact: <sip:" + userName + "@" + mylocalIp + ":5780>\r\n"
              + "Expires: 3600\r\n"
              + "Supported: 100rel\r\n"
              + "Allow: INVITE, ACK, CANCEL, OPTIONS, BYE\r\n"
              + "Max-Forwards: 70\r\n"
              + "Content-Length: 0\r\n"
              + authPrefix + @"Authorization: Digest username=""" + userName + @""", realm=""" + realm + @""", nonce=""" + nonce + @""", response=""" + authresponse + @""", uri=""sip:" + sipProviderhost + ":5060\", algorithm=MD5\r\n"
              + "\r\n";

            return message;

        }


        private string GetRealm(string message)
        {
            string res = null;
            string pattern = "realm=\"([^\\\"]+)\"";

            Regex rgx = new Regex(pattern, RegexOptions.IgnoreCase);
            MatchCollection matches = rgx.Matches(message);
            if (matches != null && matches.Count > 0)
            {
                res = matches[0].ToString();
                res = res.Replace("realm=", "");
                res = res.Replace("\"", "");
            }
            return res;

        }

        private string GetNonce(string message)
        {
            string nonce = null;
            string pattern = "nonce=\"([^\\\"]+)\"";

            Regex rgx = new Regex(pattern, RegexOptions.IgnoreCase);
            MatchCollection matches = rgx.Matches(message);
            if (matches != null && matches.Count > 0)
            {
                nonce = matches[0].ToString();
                nonce = nonce.Replace("nonce=", "");
                nonce = nonce.Replace("\"", "");
            }
            return nonce;
        }

        private string GetAuthResponse(string nonce, string realm)
        {
            string authres = null;

            string A1 = GetHashString(string.Format("{0}:{1}:{2}", authname, realm, password));
            string A2 = GetHashString("REGISTER:sip:" + sipProviderhost + ":5060");
            authres = GetHashString(string.Format("{0}:{1}:{2}", A1, nonce, A2));

            return authres;
        }


        private string GetHashString(string key)
        {
            string encoded = MD5Core.GetHashString(key).ToLower();

            return encoded;
        }


        private string GetMyip()
        {
            MyIPAddress finder = new MyIPAddress();
            IPAddress resaddr = null;
            finder.Find((address) =>
                            {
                                resaddr = address;
                            });
            if (resaddr != null)
            {
                return resaddr.ToString();
            }
            return null;
        }
    }
}
